import { Component, Host, Event, h, State, Listen, Prop, Watch, } from '@stencil/core';
import { secondsToHours } from '../../../utils/utils';
export class PeVideoControls {
  constructor() {
    this.speedIndex = 0;
    this.isMuted = false;
    this.isPlaying = false;
    /**
     * Determines current value of fullscreen toggle.
     */
    this.isFullscreen = false;
    /**
     * Determines current time of progress bar.
     */
    this.currentTime = 0.0;
    /**
     * Determines total duration of progress bar.
     */
    this.totalDuration = 0.0;
    /**
     * Determines available playback speeds.
     */
    this.availableSpeeds = [1, 2, 4, 6];
  }
  connectedCallback() {
    this.handleClickPlay = this.handleClickPlay.bind(this);
    this.handleClickRewind = this.handleClickRewind.bind(this);
    this.handleClickSpeed = this.handleClickSpeed.bind(this);
    this.handleInputSeek = this.handleInputSeek.bind(this);
    this.handleClickFullscreen = this.handleClickFullscreen.bind(this);
    this.handleClickMute = this.handleClickMute.bind(this);
  }
  currentTimeChanged() {
    this.updateInputStyle();
  }
  totalDurationChanged() {
    this.updateInputStyle();
  }
  availableSpeedsChanged(newAvailableSpeeds) {
    if (typeof newAvailableSpeeds === 'string') {
      this.availableSpeeds = JSON.parse(newAvailableSpeeds);
    }
    else if (newAvailableSpeeds === null ||
      newAvailableSpeeds === undefined) {
      this.availableSpeeds = [1, 2, 4, 6];
    }
    else {
      this.availableSpeeds = newAvailableSpeeds;
    }
  }
  peVideoPlayHandler(isPlaying) {
    this.peVideoPlay.emit(isPlaying);
  }
  peVideoRewindHandler(time) {
    this.peVideoRewind.emit(time);
  }
  peVideoSpeedHandler(speed) {
    this.peVideoSpeed.emit(speed);
  }
  peVideoMuteHandler(isMuted) {
    this.peVideoMute.emit(isMuted);
  }
  peVideoSeekHandler(time) {
    this.peVideoSeek.emit(time);
  }
  peVideoFullscreenHandler(isFullscreen) {
    this.peVideoFullscreen.emit(isFullscreen);
  }
  fullscreenchangeHandler() {
    if (document.fullscreenElement == null) {
      this.peVideoFullscreen.emit(false);
    }
  }
  // Click handlers
  handleClickMute() {
    this.isMuted = !this.isMuted;
    this.peVideoMuteHandler(this.isMuted);
  }
  handleClickRewind(event) {
    const time = event.target.dataset['time'];
    this.peVideoRewindHandler(+time);
  }
  handleClickPlay() {
    this.isPlaying = !this.isPlaying;
    this.peVideoPlayHandler(this.isPlaying);
  }
  handleClickFullscreen() {
    this.peVideoFullscreenHandler(!this.isFullscreen);
  }
  handleClickSpeed() {
    this.speedIndex = (this.speedIndex + 1) % this.availableSpeeds.length;
    this.peVideoSpeedHandler(this.availableSpeeds[this.speedIndex]);
  }
  // Input handlers
  handleInputSeek(event) {
    const time = event.target.value;
    this.peVideoSeekHandler(time);
  }
  updateInputStyle() {
    // to style progress of range input
    if (this.totalDuration !== 0.0) {
      this.inputElement.style.setProperty('--value', this.inputElement.value);
      this.inputElement.style.setProperty('--max', ['', '0'].includes(this.inputElement.max)
        ? '100'
        : this.inputElement.max);
      this.inputElement.style.setProperty('--min', this.inputElement.min == '' ? '0' : this.inputElement.min);
      this.inputElement.addEventListener('input', () => this.inputElement.style.setProperty('--value', this.inputElement.value));
    }
  }
  componentWillLoad() {
    this.availableSpeedsChanged(this.availableSpeeds);
  }
  componentDidLoad() {
    this.updateInputStyle();
  }
  componentDidUpdate() {
    this.updateInputStyle();
  }
  render() {
    return (h(Host, null,
      h("div", { id: "controls-container" },
        h("span", null,
          h("pe-button", { id: "mute-btn", iconName: this.isMuted ? 'volumeOff' : 'volumeUp', color: "primary", variant: "filled", circled: true, iconOnly: true, onClick: this.handleClickMute })),
        h("span", { class: "btn-group" },
          h("pe-button", { id: "rewind-btn", iconName: "rewindX15", color: "primary", variant: "filled", circled: true, iconOnly: true, "data-time": -15, onClick: this.handleClickRewind }),
          h("pe-button", { id: "play-btn", iconName: this.isPlaying ? 'pause' : 'play', color: "player", variant: "filled", circled: true, iconOnly: true, onClick: this.handleClickPlay }),
          h("pe-button", { id: "forward-btn", iconName: "forwardX15", color: "primary", variant: "filled", circled: true, iconOnly: true, "data-time": 15, onClick: this.handleClickRewind }),
          h("pe-button", { id: "speed-btn", iconName: `speedX${this.availableSpeeds[this.speedIndex].toString()}`, color: "primary", variant: "filled", circled: true, iconOnly: true, onClick: this.handleClickSpeed })),
        h("span", null,
          h("pe-button", { id: "fullscreen-btn", iconName: this.isFullscreen ? 'fullscreenExit' : 'fullscreen', color: "primary", variant: "filled", circled: true, iconOnly: true, onClick: this.handleClickFullscreen }))),
      h("div", { id: "progress-container" },
        h("input", { ref: el => (this.inputElement = el), onInput: this.handleInputSeek, type: "range", max: this.totalDuration, value: this.currentTime })),
      h("div", { id: "time-container" },
        h("span", { id: "current-time-text" }, secondsToHours(this.currentTime)),
        h("span", { id: "total-duration-text" }, secondsToHours(this.totalDuration)))));
  }
  static get is() { return "pe-video-controls"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["pe-video-controls.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pe-video-controls.css"]
  }; }
  static get properties() { return {
    "isFullscreen": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines current value of fullscreen toggle."
      },
      "attribute": "is-fullscreen",
      "reflect": false,
      "defaultValue": "false"
    },
    "currentTime": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines current time of progress bar."
      },
      "attribute": "current-time",
      "reflect": false,
      "defaultValue": "0.0"
    },
    "totalDuration": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines total duration of progress bar."
      },
      "attribute": "total-duration",
      "reflect": false,
      "defaultValue": "0.0"
    },
    "availableSpeeds": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "number[]",
        "resolved": "number[]",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Determines available playback speeds."
      },
      "defaultValue": "[1, 2, 4, 6]"
    }
  }; }
  static get states() { return {
    "speedIndex": {},
    "isMuted": {},
    "isPlaying": {}
  }; }
  static get events() { return [{
      "method": "peVideoPlay",
      "name": "peVideoPlay",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when play/pause button clicked."
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }, {
      "method": "peVideoRewind",
      "name": "peVideoRewind",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when rewind or forward butons clicked."
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "peVideoSpeed",
      "name": "peVideoSpeed",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when playback rate changed."
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "peVideoMute",
      "name": "peVideoMute",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when mute button clicked."
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }, {
      "method": "peVideoSeek",
      "name": "peVideoSeek",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when progress bar moved."
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "peVideoFullscreen",
      "name": "peVideoFullscreen",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when fullscreen toggle clicked."
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get watchers() { return [{
      "propName": "currentTime",
      "methodName": "currentTimeChanged"
    }, {
      "propName": "totalDuration",
      "methodName": "totalDurationChanged"
    }, {
      "propName": "availableSpeeds",
      "methodName": "availableSpeedsChanged"
    }]; }
  static get listeners() { return [{
      "name": "fullscreenchange",
      "method": "fullscreenchangeHandler",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}
