import { Component, Host, Prop, Watch, h, Method, Event, } from '@stencil/core';
export class PeVideo {
  constructor() {
    /**
     * Determines if this is the component responsible for bubble events.
     */
    this.isEventSource = false;
    /**
     * Determines media playback.
     * Triggers play() or pause() on HTMLMediaElement.
     */
    this.isPlaying = false;
    /**
     * Determines audio is muted or not.
     * Sets muted property of HTMLMediaElement.
     */
    this.isMuted = false;
    /**
     * Determines playback speed for video.
     * Sets playbackRate of HTMLMediaElement.
     */
    this.playbackRate = 1.0;
  }
  connectedCallback() {
    this.handleClick = this.handleClick.bind(this);
    this.handleWaiting = this.handleWaiting.bind(this);
    this.handleCanPlayThrough = this.handleCanPlayThrough.bind(this);
    this.handleTimeUpdate = this.handleTimeUpdate.bind(this);
    this.handleLoadedData = this.handleLoadedData.bind(this);
    this.handleEnded = this.handleEnded.bind(this);
    this.handleSuspend = this.handleSuspend.bind(this);
    this.handleStalled = this.handleStalled.bind(this);
  }
  componentDidLoad() {
    // Setting default props
    this.videoElement.muted = this.isMuted;
    this.videoElement.playbackRate = this.playbackRate;
  }
  async handleSourceChanged(newValue) {
    this.source = newValue;
    await this.videoElement.load();
    if (this.isPlaying && this.videoElement.paused)
      await this.videoElement.play();
    this.peVideoSourceChanged.emit(this.name);
  }
  async watchIsPlayingHandler(newIsPlaying) {
    if (newIsPlaying == false && !this.videoElement.paused) {
      await this.videoElement.pause();
    }
    else {
      if (this.videoElement.ended)
        return;
      if (this.videoElement.paused)
        await this.videoElement.play();
      // Make sure playback rate preserved between different video sources
      if (this.videoElement.playbackRate != this.playbackRate)
        this.videoElement.playbackRate = this.playbackRate;
    }
  }
  watchIsMutedHandler(newIsMuted) {
    this.videoElement.muted = newIsMuted;
  }
  watchPlaybackRateHandler(newPlaybackRate) {
    this.videoElement.playbackRate = newPlaybackRate;
  }
  peVideoTimeUpdateHandler(event) {
    this.peVideoTimeUpdate.emit(event);
  }
  peVideoLoadedDataHandler(event) {
    this.peVideoLoadedData.emit(event);
  }
  async peVideoCanPlayThroughHandler(event) {
    this.peVideoCanPlayThrough.emit(event);
  }
  async peVideoOnWaitingHandler(event) {
    this.peVideoOnWaiting.emit(event);
  }
  async peVideoClickHandler(event) {
    this.peVideoClick.emit(event);
  }
  async peVideoEndedHandler(event) {
    this.peVideoEnded.emit(event);
  }
  /**
   * @param time
   */
  async seek(time) {
    this.videoElement.classList.add('loading');
    this.videoElement.currentTime = this.videoElement.currentTime + time;
  }
  /**
   * @param time
   */
  async setCurrentTime(time) {
    this.videoElement.classList.add('loading');
    this.videoElement.currentTime = time;
  }
  handleClick(event) {
    this.peVideoClickHandler(event);
  }
  handleWaiting(event) {
    this.videoElement.classList.add('loading'); // Blurs video element while its loading
    this.peVideoOnWaitingHandler(event);
  }
  handleCanPlayThrough(event) {
    this.videoElement.classList.remove('loading'); // Removes blur from loaded video element
    this.peVideoCanPlayThroughHandler(event);
  }
  handleTimeUpdate(event) {
    this.peVideoTimeUpdateHandler(event);
  }
  handleLoadedData(event) {
    this.peVideoLoadedDataHandler(event);
  }
  handleEnded(event) {
    this.peVideoEndedHandler(event);
  }
  async handleSuspend() {
    this.fixStuckVideo();
  }
  async handleStalled() {
    this.fixStuckVideo();
  }
  async fixStuckVideo() {
    const video = this.videoElement;
    if (video.seeking && video.paused) {
      if (video.networkState == 1 && video.readyState == 1) {
        setTimeout(async (_this) => {
          if (!video.ended &&
            video.paused &&
            video.networkState <= 2 &&
            video.readyState <= 1) {
            await video.load();
            await video.play();
          }
        }, 5000, this);
      }
    }
  }
  render() {
    return (h(Host, { onClick: this.handleClick },
      this.tag && h("div", { class: "tag" }, this.tag),
      h("video", { id: this.name, part: "video", preload: "auto", onWaiting: this.handleWaiting, onCanPlayThrough: this.handleCanPlayThrough, onTimeUpdate: this.isEventSource == true ? this.handleTimeUpdate : null, onLoadedData: this.isEventSource == true ? this.handleLoadedData : null, onEnded: this.handleEnded, onSuspend: this.handleSuspend, onStalled: this.handleStalled, src: this.source, ref: el => (this.videoElement = el) })));
  }
  static get is() { return "pe-video"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["pe-video.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["pe-video.css"]
  }; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Unique name identifies video"
      },
      "attribute": "name",
      "reflect": true
    },
    "source": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines video source url.\nExample public url: https://components-staging.proctorexam.com/_testing-assets/landscape_sample.webm"
      },
      "attribute": "source",
      "reflect": false
    },
    "tag": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Overlay text will be shown on top of video"
      },
      "attribute": "tag",
      "reflect": false
    },
    "isEventSource": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines if this is the component responsible for bubble events."
      },
      "attribute": "is-event-source",
      "reflect": false,
      "defaultValue": "false"
    },
    "isPlaying": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines media playback.\nTriggers play() or pause() on HTMLMediaElement."
      },
      "attribute": "is-playing",
      "reflect": true,
      "defaultValue": "false"
    },
    "isMuted": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines audio is muted or not.\nSets muted property of HTMLMediaElement."
      },
      "attribute": "is-muted",
      "reflect": false,
      "defaultValue": "false"
    },
    "playbackRate": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines playback speed for video.\nSets playbackRate of HTMLMediaElement."
      },
      "attribute": "playback-rate",
      "reflect": false,
      "defaultValue": "1.0"
    }
  }; }
  static get events() { return [{
      "method": "peVideoTimeUpdate",
      "name": "peVideoTimeUpdate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Bubbles timeUpdate event of HTMLMediaElement"
      },
      "complexType": {
        "original": "CustomEvent",
        "resolved": "CustomEvent<any>",
        "references": {
          "CustomEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "peVideoLoadedData",
      "name": "peVideoLoadedData",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Bubbles loadedData event of HTMLMediaElement"
      },
      "complexType": {
        "original": "CustomEvent",
        "resolved": "CustomEvent<any>",
        "references": {
          "CustomEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "peVideoCanPlayThrough",
      "name": "peVideoCanPlayThrough",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Bubbles canPlayThrough event of HTMLMediaElement"
      },
      "complexType": {
        "original": "CustomEvent",
        "resolved": "CustomEvent<any>",
        "references": {
          "CustomEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "peVideoOnWaiting",
      "name": "peVideoOnWaiting",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Bubbles onWaiting event of HTMLMediaElement"
      },
      "complexType": {
        "original": "CustomEvent",
        "resolved": "CustomEvent<any>",
        "references": {
          "CustomEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "peVideoSourceChanged",
      "name": "peVideoSourceChanged",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits when video source changed."
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }, {
      "method": "peVideoClick",
      "name": "peVideoClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Bubbles click event of HTMLMediaElement"
      },
      "complexType": {
        "original": "CustomEvent",
        "resolved": "CustomEvent<any>",
        "references": {
          "CustomEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "peVideoEnded",
      "name": "peVideoEnded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Bubbles video ended event of HTMLMediaElement"
      },
      "complexType": {
        "original": "CustomEvent",
        "resolved": "CustomEvent<any>",
        "references": {
          "CustomEvent": {
            "location": "global"
          }
        }
      }
    }]; }
  static get methods() { return {
    "seek": {
      "complexType": {
        "signature": "(time: number) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "time"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "param",
            "text": "time"
          }]
      }
    },
    "setCurrentTime": {
      "complexType": {
        "signature": "(time: number) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "time"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "param",
            "text": "time"
          }]
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "source",
      "methodName": "handleSourceChanged"
    }, {
      "propName": "isPlaying",
      "methodName": "watchIsPlayingHandler"
    }, {
      "propName": "isMuted",
      "methodName": "watchIsMutedHandler"
    }, {
      "propName": "playbackRate",
      "methodName": "watchPlaybackRateHandler"
    }]; }
}
