import { HTMLElement, createEvent, h, Host, proxyCustomElement } from '@stencil/core/internal/client';

const peVideoCss = ":host{display:block;position:relative}video.loading{-webkit-filter:blur(10px);filter:blur(10px)}video{width:100%;height:100%;background-color:black}.tag{display:-ms-flexbox;display:flex;-ms-flex-pack:center;justify-content:center;-ms-flex-align:center;align-items:center;height:28px;position:absolute;left:0;bottom:0;z-index:2;margin:12px;padding:0px 12px;background-color:#f3f3f3;border:1px solid #cdcdcd;-webkit-box-shadow:0.5px 1px 2px rgba(0, 0, 0, 0.2);box-shadow:0.5px 1px 2px rgba(0, 0, 0, 0.2);border-radius:8px;cursor:default;color:#4d4d4d;font-weight:500;font-size:12px;line-height:18px}";

let PeVideo = class extends HTMLElement {
  constructor() {
    super();
    this.__registerHost();
    this.__attachShadow();
    this.peVideoTimeUpdate = createEvent(this, "peVideoTimeUpdate", 7);
    this.peVideoLoadedData = createEvent(this, "peVideoLoadedData", 7);
    this.peVideoCanPlayThrough = createEvent(this, "peVideoCanPlayThrough", 7);
    this.peVideoOnWaiting = createEvent(this, "peVideoOnWaiting", 7);
    this.peVideoSourceChanged = createEvent(this, "peVideoSourceChanged", 7);
    this.peVideoClick = createEvent(this, "peVideoClick", 7);
    this.peVideoEnded = createEvent(this, "peVideoEnded", 7);
    /**
     * Determines if this is the component responsible for bubble events.
     */
    this.isEventSource = false;
    /**
     * Determines media playback.
     * Triggers play() or pause() on HTMLMediaElement.
     */
    this.isPlaying = false;
    /**
     * Determines audio is muted or not.
     * Sets muted property of HTMLMediaElement.
     */
    this.isMuted = false;
    /**
     * Determines playback speed for video.
     * Sets playbackRate of HTMLMediaElement.
     */
    this.playbackRate = 1.0;
  }
  connectedCallback() {
    this.handleClick = this.handleClick.bind(this);
    this.handleWaiting = this.handleWaiting.bind(this);
    this.handleCanPlayThrough = this.handleCanPlayThrough.bind(this);
    this.handleTimeUpdate = this.handleTimeUpdate.bind(this);
    this.handleLoadedData = this.handleLoadedData.bind(this);
    this.handleEnded = this.handleEnded.bind(this);
    this.handleSuspend = this.handleSuspend.bind(this);
    this.handleStalled = this.handleStalled.bind(this);
  }
  componentDidLoad() {
    // Setting default props
    this.videoElement.muted = this.isMuted;
    this.videoElement.playbackRate = this.playbackRate;
  }
  async handleSourceChanged(newValue) {
    this.source = newValue;
    await this.videoElement.load();
    if (this.isPlaying && this.videoElement.paused)
      await this.videoElement.play();
    this.peVideoSourceChanged.emit(this.name);
  }
  async watchIsPlayingHandler(newIsPlaying) {
    if (newIsPlaying == false && !this.videoElement.paused) {
      await this.videoElement.pause();
    }
    else {
      if (this.videoElement.ended)
        return;
      if (this.videoElement.paused)
        await this.videoElement.play();
      // Make sure playback rate preserved between different video sources
      if (this.videoElement.playbackRate != this.playbackRate)
        this.videoElement.playbackRate = this.playbackRate;
    }
  }
  watchIsMutedHandler(newIsMuted) {
    this.videoElement.muted = newIsMuted;
  }
  watchPlaybackRateHandler(newPlaybackRate) {
    this.videoElement.playbackRate = newPlaybackRate;
  }
  peVideoTimeUpdateHandler(event) {
    this.peVideoTimeUpdate.emit(event);
  }
  peVideoLoadedDataHandler(event) {
    this.peVideoLoadedData.emit(event);
  }
  async peVideoCanPlayThroughHandler(event) {
    this.peVideoCanPlayThrough.emit(event);
  }
  async peVideoOnWaitingHandler(event) {
    this.peVideoOnWaiting.emit(event);
  }
  async peVideoClickHandler(event) {
    this.peVideoClick.emit(event);
  }
  async peVideoEndedHandler(event) {
    this.peVideoEnded.emit(event);
  }
  /**
   * @param time
   */
  async seek(time) {
    this.videoElement.classList.add('loading');
    this.videoElement.currentTime = this.videoElement.currentTime + time;
  }
  /**
   * @param time
   */
  async setCurrentTime(time) {
    this.videoElement.classList.add('loading');
    this.videoElement.currentTime = time;
  }
  handleClick(event) {
    this.peVideoClickHandler(event);
  }
  handleWaiting(event) {
    this.videoElement.classList.add('loading'); // Blurs video element while its loading
    this.peVideoOnWaitingHandler(event);
  }
  handleCanPlayThrough(event) {
    this.videoElement.classList.remove('loading'); // Removes blur from loaded video element
    this.peVideoCanPlayThroughHandler(event);
  }
  handleTimeUpdate(event) {
    this.peVideoTimeUpdateHandler(event);
  }
  handleLoadedData(event) {
    this.peVideoLoadedDataHandler(event);
  }
  handleEnded(event) {
    this.peVideoEndedHandler(event);
  }
  async handleSuspend() {
    this.fixStuckVideo();
  }
  async handleStalled() {
    this.fixStuckVideo();
  }
  async fixStuckVideo() {
    const video = this.videoElement;
    if (video.seeking && video.paused) {
      if (video.networkState == 1 && video.readyState == 1) {
        setTimeout(async (_this) => {
          if (!video.ended &&
            video.paused &&
            video.networkState <= 2 &&
            video.readyState <= 1) {
            await video.load();
            await video.play();
          }
        }, 5000, this);
      }
    }
  }
  render() {
    return (h(Host, { onClick: this.handleClick }, this.tag && h("div", { class: "tag" }, this.tag), h("video", { id: this.name, part: "video", preload: "auto", onWaiting: this.handleWaiting, onCanPlayThrough: this.handleCanPlayThrough, onTimeUpdate: this.isEventSource == true ? this.handleTimeUpdate : null, onLoadedData: this.isEventSource == true ? this.handleLoadedData : null, onEnded: this.handleEnded, onSuspend: this.handleSuspend, onStalled: this.handleStalled, src: this.source, ref: el => (this.videoElement = el) })));
  }
  static get watchers() { return {
    "source": ["handleSourceChanged"],
    "isPlaying": ["watchIsPlayingHandler"],
    "isMuted": ["watchIsMutedHandler"],
    "playbackRate": ["watchPlaybackRateHandler"]
  }; }
  static get style() { return peVideoCss; }
};
PeVideo = /*@__PURE__*/ proxyCustomElement(PeVideo, [1, "pe-video", {
    "name": [513],
    "source": [1025],
    "tag": [1],
    "isEventSource": [1028, "is-event-source"],
    "isPlaying": [1540, "is-playing"],
    "isMuted": [1028, "is-muted"],
    "playbackRate": [1026, "playback-rate"],
    "seek": [64],
    "setCurrentTime": [64]
  }]);
function defineCustomElement() {
  if (typeof customElements === "undefined") {
    return;
  }
  const components = ["pe-video"];
  components.forEach(tagName => { switch (tagName) {
    case "pe-video":
      if (!customElements.get(tagName)) {
        customElements.define(tagName, PeVideo);
      }
      break;
  } });
}
defineCustomElement();

export { PeVideo as P, defineCustomElement as d };
